@@ -139,8 +139,17 @@ static const Itdb_IpodInfo ipod_info_table [] = {
 
     /* Shuffle / Eigth Generation */
     /* Bar, button-less, speaking */
-    {"B867", 4,  ITDB_IPOD_MODEL_SHUFFLE_SILVER, ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
-    {"C164", 4,  ITDB_IPOD_MODEL_SHUFFLE_BLACK,  ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C306", 2,  ITDB_IPOD_MODEL_SHUFFLE_SILVER,    ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C323", 2,  ITDB_IPOD_MODEL_SHUFFLE_BLACK,     ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C381", 2,  ITDB_IPOD_MODEL_SHUFFLE_GREEN,     ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C384", 2,  ITDB_IPOD_MODEL_SHUFFLE_BLUE,      ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C387", 2,  ITDB_IPOD_MODEL_SHUFFLE_PINK,      ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"B867", 4,  ITDB_IPOD_MODEL_SHUFFLE_SILVER,    ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C164", 4,  ITDB_IPOD_MODEL_SHUFFLE_BLACK,     ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C303", 4,  ITDB_IPOD_MODEL_SHUFFLE_STAINLESS, ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C307", 4,  ITDB_IPOD_MODEL_SHUFFLE_GREEN,     ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C328", 4,  ITDB_IPOD_MODEL_SHUFFLE_BLUE,      ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
+    {"C331", 4,  ITDB_IPOD_MODEL_SHUFFLE_PINK,      ITDB_IPOD_GENERATION_SHUFFLE_4, 3},
 
     /* Nano / Fifth Generation (first nano generation) */
     /* Buttons are integrated into the "touch wheel". */
@@ -290,6 +299,9 @@ static const Itdb_IpodInfo ipod_info_table [] = {
     {"C131",  16, ITDB_IPOD_MODEL_IPHONE_BLACK,   ITDB_IPOD_GENERATION_IPHONE_3, 50},
     {"C133",  32, ITDB_IPOD_MODEL_IPHONE_BLACK,   ITDB_IPOD_GENERATION_IPHONE_3, 50},
     {"C134",  32, ITDB_IPOD_MODEL_IPHONE_WHITE,   ITDB_IPOD_GENERATION_IPHONE_3, 50},
+    /* iPhone 4G */
+    {"C603",  16, ITDB_IPOD_MODEL_IPHONE_BLACK,   ITDB_IPOD_GENERATION_IPHONE_4, 50},
+    {"C605",  32, ITDB_IPOD_MODEL_IPHONE_BLACK,   ITDB_IPOD_GENERATION_IPHONE_4, 50},
 
     /* iPad */
     {"B292",  16, ITDB_IPOD_MODEL_IPAD,           ITDB_IPOD_GENERATION_IPAD_1,   50},
@@ -766,6 +778,7 @@ static const ItdbSerialToModel serial_to_model_mapping[] = {
     { "726", "C037" }, /* 8GB Blue Nano 5g */
     { "72A", "C040" }, /* 8GB Green Nano 5g */
     { "72F", "C046" }, /* 8GB Orange Nano 5g */
+    { "72K", "C049" }, /* 8GB Red Nano 5g */
     { "72L", "C050" }, /* 8GB Pink Nano 5g */
 
     { "72Q", "C060" }, /* 16GB Silver Nano 5g */
@@ -778,8 +791,17 @@ static const ItdbSerialToModel serial_to_model_mapping[] = {
     { "73A", "C074" }, /* 16GB Red Nano 5g */
     { "73B", "C075" }, /* 16GB Pink Nano 5g */
 
+    { "A1S", "C306" }, /* 2GB Silver Shuffle 4g */
+    { "A78", "C323" }, /* 2GB Black Shuffle 4g */
+    { "ALB", "C381" }, /* 2GB Green Shuffle 4g */
+    { "ALD", "C384" }, /* 2GB Blue Shuffle 4g */
+    { "ALG", "C387" }, /* 2GB Pink Shuffle 4g */
     { "4NZ", "B867" }, /* 4GB Silver Shuffle 4g */
     { "891", "C164" }, /* 4GB Black Shuffle 4g */
+    { "A1L", "C303" }, /* 4GB Stainless Shuffle 4g */
+    { "A1U", "C307" }, /* 4GB Green Shuffle 4g */
+    { "A7B", "C328" }, /* 4GB Blue Shuffle 4g */
+    { "A7D", "C331" }, /* 4GB Pink Shuffle 4g */
 
     { "W4N", "A623" }, /* 8GB Silver iPod Touch (1st gen) */
     { "W4T", "A627" }, /* 16GB Silver iPod Touch (1st gen) */
@@ -798,7 +820,11 @@ static const ItdbSerialToModel serial_to_model_mapping[] = {
     { "3NP", "C131" }, /* 16GB Black iPhone 3GS */
     { "3NR", "C133" }, /* 32GB Black iPhone 3GS */
     { "3NS", "C134" }, /* 32GB White iPhone 3GS */
+    { "A4S", "C603" }, /* 16GB Black iPhone 4G */
+    { "A4T", "C605" }, /* 32GB Black iPhone 4G */
 
+    { "Z38", "B292" }, /* 16GB iPad with Wifi */
+    { "Z39", "B293" }, /* 32GB iPad with Wifi */
     { "Z3A", "B294" }, /* 64GB iPad with Wifi */
     { NULL ,  NULL  }
 };
@@ -1318,6 +1344,7 @@ itdb_device_supports_sqlite_db (const Itdb_Device *device)
 	    case ITDB_IPOD_GENERATION_IPHONE_1:
 	    case ITDB_IPOD_GENERATION_IPHONE_2:
 	    case ITDB_IPOD_GENERATION_IPHONE_3:
+	    case ITDB_IPOD_GENERATION_IPHONE_4:
 	    case ITDB_IPOD_GENERATION_IPAD_1:
 		/* FIXME: needs to check firmware version */
 		return TRUE;
@@ -1378,6 +1405,7 @@ itdb_device_supports_sparse_artwork (const Itdb_Device *device)
             case ITDB_IPOD_GENERATION_IPHONE_1:
             case ITDB_IPOD_GENERATION_IPHONE_2:
             case ITDB_IPOD_GENERATION_IPHONE_3:
+            case ITDB_IPOD_GENERATION_IPHONE_4:
             case ITDB_IPOD_GENERATION_IPAD_1:
                 supports_sparse_artwork = TRUE;
                 break;
@@ -1687,6 +1715,7 @@ gboolean itdb_device_supports_video (const Itdb_Device *device)
 	case ITDB_IPOD_GENERATION_IPHONE_1:
 	case ITDB_IPOD_GENERATION_IPHONE_2:
 	case ITDB_IPOD_GENERATION_IPHONE_3:
+	case ITDB_IPOD_GENERATION_IPHONE_4:
 	case ITDB_IPOD_GENERATION_IPAD_1:
             return TRUE;
     }
@@ -1778,6 +1807,8 @@ ItdbChecksumType itdb_device_get_checksum_type (const Itdb_Device *device)
 		return ITDB_CHECKSUM_HASH58;
 	    case 4:
 		return ITDB_CHECKSUM_HASH72;
+	    case 5:
+		return ITDB_CHECKSUM_HASHAB;
 	    default:
 		return ITDB_CHECKSUM_UNKNOWN;
 	}
@@ -1803,9 +1834,12 @@ ItdbChecksumType itdb_device_get_checksum_type (const Itdb_Device *device)
 	    case ITDB_IPOD_GENERATION_IPHONE_1:
 	    case ITDB_IPOD_GENERATION_IPHONE_2:
 	    case ITDB_IPOD_GENERATION_IPHONE_3:
-	    case ITDB_IPOD_GENERATION_IPAD_1:
 		return ITDB_CHECKSUM_HASH72;
 
+	    case ITDB_IPOD_GENERATION_IPAD_1:
+	    case ITDB_IPOD_GENERATION_IPHONE_4:
+		return ITDB_CHECKSUM_HASHAB;
+
 	    case ITDB_IPOD_GENERATION_UNKNOWN:
 	    case ITDB_IPOD_GENERATION_FIRST:
 	    case ITDB_IPOD_GENERATION_SECOND:
@@ -1842,6 +1876,7 @@ G_GNUC_INTERNAL gboolean itdb_device_write_checksum (Itdb_Device *device,
 	    return itdb_hash58_write_hash (device, itdb_data, itdb_len, error);
 	case ITDB_CHECKSUM_HASH72:
 	    return itdb_hash72_write_hash (device, itdb_data, itdb_len, error);
+        case ITDB_CHECKSUM_HASHAB:
 	case ITDB_CHECKSUM_UNKNOWN:
             g_set_error (error, 0, -1, "Unsupported checksum type");
 	    return FALSE;
@@ -2043,6 +2078,7 @@ itdb_device_supports_podcast (const Itdb_Device *device)
 	    case ITDB_IPOD_GENERATION_IPHONE_1:
 	    case ITDB_IPOD_GENERATION_IPHONE_2:
 	    case ITDB_IPOD_GENERATION_IPHONE_3:
+	    case ITDB_IPOD_GENERATION_IPHONE_4:
 	    case ITDB_IPOD_GENERATION_IPAD_1:
 		return TRUE;
 	}
@@ -2086,6 +2122,7 @@ gboolean itdb_device_is_shuffle (const Itdb_Device *device)
         case ITDB_IPOD_GENERATION_IPHONE_1:
         case ITDB_IPOD_GENERATION_IPHONE_2:
         case ITDB_IPOD_GENERATION_IPHONE_3:
+        case ITDB_IPOD_GENERATION_IPHONE_4:
         case ITDB_IPOD_GENERATION_IPAD_1:
             return FALSE;
         case ITDB_IPOD_GENERATION_SHUFFLE_1:
@@ -2136,9 +2173,67 @@ gboolean itdb_device_is_iphone_family (const Itdb_Device *device)
 	    case ITDB_IPOD_GENERATION_IPHONE_1:
 	    case ITDB_IPOD_GENERATION_IPHONE_2:
 	    case ITDB_IPOD_GENERATION_IPHONE_3:
+	    case ITDB_IPOD_GENERATION_IPHONE_4:
 	    case ITDB_IPOD_GENERATION_IPAD_1:
 		return TRUE;
 	}
 	g_return_val_if_reached (FALSE);
     }
 }
+
+enum ItdbShadowDBVersion itdb_device_get_shadowdb_version (const Itdb_Device *device)
+{
+    int version;
+
+    version = 0;
+    if (device->sysinfo_extended != NULL) {
+	version = itdb_sysinfo_properties_get_shadow_db_version (device->sysinfo_extended);
+    }
+
+    if (version == 0) {
+	const Itdb_IpodInfo *info;
+
+	info = itdb_device_get_ipod_info (device);
+	switch (info->ipod_generation) {
+	    case ITDB_IPOD_GENERATION_UNKNOWN:
+	    case ITDB_IPOD_GENERATION_FIRST:
+	    case ITDB_IPOD_GENERATION_SECOND:
+	    case ITDB_IPOD_GENERATION_THIRD:
+	    case ITDB_IPOD_GENERATION_MOBILE:
+	    case ITDB_IPOD_GENERATION_FOURTH:
+	    case ITDB_IPOD_GENERATION_PHOTO:
+	    case ITDB_IPOD_GENERATION_MINI_1:
+	    case ITDB_IPOD_GENERATION_MINI_2:
+	    case ITDB_IPOD_GENERATION_NANO_1:
+	    case ITDB_IPOD_GENERATION_NANO_2:
+	    case ITDB_IPOD_GENERATION_NANO_3:
+	    case ITDB_IPOD_GENERATION_NANO_4:
+	    case ITDB_IPOD_GENERATION_NANO_5:
+	    case ITDB_IPOD_GENERATION_VIDEO_1:
+	    case ITDB_IPOD_GENERATION_VIDEO_2:
+	    case ITDB_IPOD_GENERATION_CLASSIC_1:
+	    case ITDB_IPOD_GENERATION_CLASSIC_2:
+	    case ITDB_IPOD_GENERATION_CLASSIC_3:
+	    case ITDB_IPOD_GENERATION_TOUCH_1:
+	    case ITDB_IPOD_GENERATION_TOUCH_2:
+	    case ITDB_IPOD_GENERATION_TOUCH_3:
+	    case ITDB_IPOD_GENERATION_IPHONE_1:
+	    case ITDB_IPOD_GENERATION_IPHONE_2:
+	    case ITDB_IPOD_GENERATION_IPHONE_3:
+	    case ITDB_IPOD_GENERATION_IPHONE_4:
+	    case ITDB_IPOD_GENERATION_IPAD_1:
+		version = ITDB_SHADOW_DB_UNKNOWN;
+		break;
+	    case ITDB_IPOD_GENERATION_SHUFFLE_1:
+	    case ITDB_IPOD_GENERATION_SHUFFLE_2:
+		version = ITDB_SHADOW_DB_V1;
+		break;
+	    case ITDB_IPOD_GENERATION_SHUFFLE_3:
+	    case ITDB_IPOD_GENERATION_SHUFFLE_4:
+		version = ITDB_SHADOW_DB_V2;
+		break;
+	}
+    }
+
+    return version;
+}
