@@ -79,7 +79,8 @@ enum _ItdbChecksumType {
     ITDB_CHECKSUM_UNKNOWN	= -1,
     ITDB_CHECKSUM_NONE		= 0,
     ITDB_CHECKSUM_HASH58	= 1,
-    ITDB_CHECKSUM_HASH72	= 2
+    ITDB_CHECKSUM_HASH72	= 2,
+    ITDB_CHECKSUM_HASHAB	= 3
 };
 
 /**
@@ -130,12 +131,12 @@ struct _Itdb_Device
  *                      (not currently used)
  * @interlaced:         If TRUE, the thumbnails are interlaced
  *                      (not currently used)
- * @align_row_bytes:    If TRUE, each pixel row must be aligned a 2-byte boundary
  * @color_adjustment:   Color adjustment for the thumbnails
  *                      (not currently used)
  * @gamma:              Gamma value for the thumbails
  *                      (not currently used)
  * @associated_format:  Unknown (not currently used)
+ * @row_bytes_alignment: Specifies the number of bytes a pixel row must be aligned to
  *
  * Structure representing the characteristics of the thumbnails to
  * write to a given .ithmb file. The format of the structure is based
@@ -153,10 +154,10 @@ struct _Itdb_ArtworkFormat {
 
         gint display_width;
         gboolean interlaced;
-        gboolean align_row_bytes;
         gint color_adjustment;
         gdouble gamma;
         gint associated_format;
+        gint row_bytes_alignment;
 };
 
 /* Error domain */
@@ -181,6 +182,7 @@ G_GNUC_INTERNAL void itdb_device_set_timezone_info (Itdb_Device *device);
 G_GNUC_INTERNAL gboolean itdb_device_is_iphone_family (const Itdb_Device *device);
 G_GNUC_INTERNAL gboolean itdb_device_is_shuffle (const Itdb_Device *device);
 G_GNUC_INTERNAL ItdbChecksumType itdb_device_get_checksum_type (const Itdb_Device *device);
+G_GNUC_INTERNAL enum ItdbShadowDBVersion itdb_device_get_shadowdb_version (const Itdb_Device *device);
 
 const Itdb_IpodInfo *
 itdb_ipod_info_from_serial (const char *serial);
