@@ -80,6 +80,7 @@ struct _SysInfoIpodProperties {
         gint oem_id;
         gint oem_u;
         gint db_version;
+        int shadowdb_version;
         char *min_build_id;
         char *language;
         gboolean voice_memos_supported;
@@ -218,6 +219,7 @@ static const DictFieldMapping sysinfo_ipod_properties_fields_mapping[] = {
     { "GamesPlatformVersion",          G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, games_platform_version) },
     { "RentalClockBias",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, rental_clock_bias) },
     { "SQLiteDB",                      G_TYPE_BOOLEAN, G_STRUCT_OFFSET (SysInfoIpodProperties, sqlite_db) },
+    { "ShadowDBVersion",               G_TYPE_INT,     G_STRUCT_OFFSET (SysInfoIpodProperties, shadowdb_version) },
     { NULL,                            G_TYPE_NONE,    0 }
 };
 
@@ -231,7 +233,9 @@ static const DictFieldMapping sysinfo_image_format_fields_mapping[] = {
 /*  { "PixelFormat",      G_TYPE_STRING,  G_STRUCT_OFFSET (Itdb_ArtworkFormat, format) },*/
     { "Interlaced",       G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, interlaced) },
     { "Crop",             G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, crop) },
-    { "AlignRowBytes",    G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, align_row_bytes) },
+/* AlignRowBytes is an older version of RowBytesAlignment, it's equivalent
+ * to a value of 4 for RowBytesAlignemnt */
+/*    { "AlignRowBytes",    G_TYPE_BOOLEAN, G_STRUCT_OFFSET (Itdb_ArtworkFormat, align_row_bytes) },*/
     { "Rotation",         G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, rotation) },
 /* BackColor needs to be converted to a gint, this is special-cased
  * in g_value_to_image_format */
@@ -239,6 +243,7 @@ static const DictFieldMapping sysinfo_image_format_fields_mapping[] = {
     { "ColorAdjustment",  G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, color_adjustment) },
     { "GammaAdjustment",  G_TYPE_DOUBLE,  G_STRUCT_OFFSET (Itdb_ArtworkFormat, gamma) },
     { "AssociatedFormat", G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, associated_format) },
+    { "RowBytesAlignment", G_TYPE_INT,     G_STRUCT_OFFSET (Itdb_ArtworkFormat, row_bytes_alignment) },
     { NULL,               G_TYPE_NONE,    0 }
 };
 
@@ -457,6 +462,14 @@ static Itdb_ArtworkFormat *g_value_to_image_format (GValue *value)
 
     dict_to_struct (dict, sysinfo_image_format_fields_mapping, img_spec);
 
+    if (get_boolean (dict, "AlignRowBytes")
+            && (img_spec->row_bytes_alignment == 0)) {
+        /* at least the nano3g has the AlignRowBytes key with no
+         * RowBytesAlignment key.
+         */
+        img_spec->row_bytes_alignment = 4;
+    }
+
     return img_spec;
 }
 
@@ -712,3 +725,10 @@ itdb_sysinfo_properties_get_db_version (const SysInfoIpodProperties *props)
     g_return_val_if_fail (props != NULL, FALSE);
     return props->db_version;
 }
+
+gint
+itdb_sysinfo_properties_get_shadow_db_version (const SysInfoIpodProperties *props)
+{
+   g_return_val_if_fail (props != NULL, 0);
+   return props->shadowdb_version;
+}
