@@ -144,6 +144,7 @@ get_model_name (const Itdb_IpodInfo *info)
 	case ITDB_IPOD_GENERATION_IPHONE_1:
 	case ITDB_IPOD_GENERATION_IPHONE_2:
 	case ITDB_IPOD_GENERATION_IPHONE_3:
+	case ITDB_IPOD_GENERATION_IPHONE_4:
 		return g_strdup ("phone");
 	case ITDB_IPOD_GENERATION_IPAD_1:
 		return g_strdup ("ipad");
@@ -216,6 +217,8 @@ get_generation (const Itdb_IpodInfo *info)
 		return 2.0;
 	case ITDB_IPOD_GENERATION_IPHONE_3:
 		return 3.0;
+	case ITDB_IPOD_GENERATION_IPHONE_4:
+		return 4.0;
 	case ITDB_IPOD_GENERATION_IPAD_1:
 		return 1.0;
 	case ITDB_IPOD_GENERATION_MOBILE:
@@ -416,6 +419,8 @@ get_icon_name (const Itdb_IpodInfo *info)
 		return g_strdup ("phone-apple-iphone-3g");
 	case ITDB_IPOD_GENERATION_IPHONE_3:
 		return g_strdup ("phone-apple-iphone-3gs");
+	case ITDB_IPOD_GENERATION_IPHONE_4:
+		return g_strdup ("phone-apple-iphone-4g");
 	case ITDB_IPOD_GENERATION_IPAD_1:
 		return g_strdup ("computer-apple-ipad");
 	case ITDB_IPOD_GENERATION_MOBILE:
@@ -603,7 +608,17 @@ static gboolean write_sysinfo_extended (const char *mountpoint,
         devdirpath = itdb_get_device_dir (mountpoint);
         /* Make sure the device dir exists (not necessarily true on
          * Shuffles */
-        g_mkdir (devdirpath, 0777);
+        if (devdirpath == NULL) {
+            gchar *itunesdirpath;
+
+            itunesdirpath = itdb_get_itunes_dir (mountpoint);
+            if (itunesdirpath == NULL) {
+                return FALSE;
+            }
+            devdirpath = g_build_filename (itunesdirpath, "Device", NULL);
+            g_free (itunesdirpath);
+            g_mkdir (devdirpath, 0777);
+        }
         filename = g_build_filename (devdirpath, "SysInfoExtended", NULL);
         g_free (devdirpath);
         if (filename == NULL) {
